% This program aims to test converse relations identification.
% for example, the BART learned weapon:spear, tree: oak. 
% test: red: color, robin:bird

clear all;
tic; 
% rng(1); % set random seed
input = 'word2vec'; %'topics'; 
ConverFlag = 1;  % 1: do converse operation; 0: use raw

alphaval = .5;   % alpha value in logistic regression with elastic net regularization. alpha = 0 is ridge regression with L2 regularization; alpha = 0 is lasso regression with L1 regularization
displayflag = 0; % 1: show some intermediate display info; 0: no show of intermediate results
datapath = '.\data\SemEval\Phase2AnswersScaled\';
num_train_pairs_pos =  20; 	% number of positive training examples for comparative relations
outputrstpath = ['.\rst\SemEval\training' num2str(num_train_pairs_pos) '\'];

% Input-specific parameters
num_sel_dims = 300;          % number of dimensions to select, or 0 if shouldn't select dimensions
use_hyperprior = true; %false;%     % whether to use hyperprior


% input dataset: SemEval
filerel = {'1a','1b','1c','1d','1e','2a','2b','2c','2d','2e','2f','2g','2h','2i','2j',...
    '3a','3b','3c','3d','3e','3f','3g','3h','4a','4b','4c','4d','4e','4f','4g','4h',...
    '5a','5b','5c','5d','5e','5f','5g','5h','5i',...
    '6a','6b','6c','6d','6e','6f','6g','6h',...
    '7a','7b','7c','7d','7e','7f','7g','7h',...
    '8a','8b','8c','8d','8e','8f','8g','8h',...
    '9a','9b','9c','9d','9e','9f','9g','9h','9i',...
    '10a','10b','10c','10d','10e','10f'};%
feature_names = filerel;
typelab = {1:5, 6:15,16:23,24:31,32:40,41:48,49:56,57:64,65:73,74:79};
numpairperrel = 1;  % select # of word pair per relation to serve as negative examples for learning other relations

testrel =  filerel;%
learnrellabel = 1:length(testrel);

% For variational method
max_iter = 500;%5000;                % maximum number of iterations
criterion = 0.00001;            % criterion for convergence


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Get input data and do pre-processing %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

numfeats = num_sel_dims;
num_pair_feats = numfeats * 2;

for relation_to_learn = 1:length(testrel)
    
    % load relation weight distribution from learning
    load([outputrstpath sprintf('%s_mumat.mat', testrel{relation_to_learn})]);
    mu_post = mumat(:,2); 
    load([outputrstpath sprintf('%s_sigma.mat', testrel{relation_to_learn})]); 
    load([outputrstpath sprintf('%s_rsttemp.mat', testrel{relation_to_learn})]);%,'imptdimindx','selobjfdim'

    % generate converse rel representation
    if ConverFlag==1
        [mu_conv,sigma_post_cov,imptdimindx_conv,dim_means_conv ]=converse_func(mumat,sigma_post,imptdimindx,num_sel_dims,dim_means);
        mu_post = mu_conv;
        sigma_post = sigma_post_cov;
        imptdimindx = imptdimindx_conv;
        dim_means = dim_means_conv;
    end;
    
    %% load human ratings
    filename = [datapath  'Phase2Answers-' filerel{relation_to_learn} '.txt'];
    fid = fopen( filename);
    data = textscan(fid, '%f %q','headerlines', 11);
    fclose(fid);

    humrating{relation_to_learn} = data{1,1};
    wordpairs = length(humrating{relation_to_learn}); % # of word-pairs from the SemEv dataset.

    %% load word vec
    datavalprot = [];
    load([datapath  'Phase2Answers-' filerel{relation_to_learn} '.txt.mat'],'allData'); 
    if size(allData,1)~=2*wordpairs   % there are some extra lines in the original mat files
        warning('more word pairs than human ratings. check the raw data file\n'); 
        allDatatemp = allData;
        allData = [];
        allData = allDatatemp(1:2*wordpairs,:);
        save([datapath  'Phase2Answers-' filerel{relation_to_learn} '.txt.mat'],'allData'); 
    end;

    datavalprot=cell2mat(allData(1:2*wordpairs,2:301));
    Gpairnum(relation_to_learn) = size(datavalprot,1)/2;  

    % generate positive pairs with converse
    if ConverFlag==0
        for si = 1:Gpairnum(relation_to_learn)
            all_pos_vecs(si,:) = [datavalprot((si-1)*2+1,:) datavalprot((si-1)*2+2,:)];
            all_pos_names{si,1} = strcat(allData{(si-1)*2+1,1}, ':', allData{(si-1)*2+2,1});
        end;
    else
        for si = 1:Gpairnum(relation_to_learn)
            all_pos_vecs(si,:) = [datavalprot((si-1)*2+2,:) datavalprot((si-1)*2+1,:)];
            all_pos_names{si,1} = strcat(allData{(si-1)*2+2,1}, ':', allData{(si-1)*2+1,1});
        end;
    end;
    
    feature_name = feature_names{relation_to_learn};
    fprintf('\n Test reverse relation of %s:', feature_name);
        
   
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Create the entire pool of possible training and test items %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    tot_num_pos_pairs = Gpairnum(relation_to_learn);
    % Test all pairs in the target relation including training pairs
    test_pos_inds = 1:tot_num_pos_pairs;
    test_data = all_pos_vecs(test_pos_inds, :);
    test_data_labels = humrating{relation_to_learn}>0; % find pairs that humans give positive typicality ratings
    test_data_names = all_pos_names(test_pos_inds, :);

    num_test_pairs = size(test_data,1);
     
        
    % test new word pairs, and generate the concatanated difference vectors
    t_data_diff = []; test_data_diff = []; t_scoreall=[];
    t_data_diff = ( test_data(:, 1:num_sel_dims)-test_data(:, num_sel_dims+1:end));          
    for ti = 1:size(t_data_diff,1)
        temp1 = t_data_diff(ti, 1:num_sel_dims);
        [sorttemp1,sortindxtemp] = sort(temp1);
        sortindxt(ti,:) = sortindxtemp;
    end;


    for ti = 1:size(test_data,1)
         test_data2raw(ti,:) = [test_data(ti,1:numfeats) test_data(ti,sortindxt(ti,:)) test_data(ti,numfeats+1:end) test_data(ti,numfeats+sortindxt(ti,:))];
    end;    

    totalobjfdim = size(test_data2raw,2); 
    selobjfdim = [imptdimindx; numfeats*2+imptdimindx];

    % use selected feature dimensions
    test_data2 = [test_data2raw(:,selobjfdim) ];   
    
    % centering
    test_data2 = test_data2 - repmat(dim_means, size(test_data2,1), 1);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Evaluate performance for test pairs and save results 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     % Temp test; drop off the covs
    
    sigma_inv_post = pinv(sigma_post);

    
    pred_prob_pos=[];
    for test = 1 : num_test_pairs
        pair_vec_pos = test_data2(test, :);
        pred_prob_pos(test,1) = compute_score(pair_vec_pos', 1, mu_post, sigma_inv_post, max_iter, criterion);
%         fprintf('%d %s %f\n',test,char(test_data_names{test,:}), pred_prob_pos(test,1) );
    end; 
    
    temp= [test_data2raw(1,selobjfdim)' pair_vec_pos' mu_post dim_means']; temp1 = [ imptdimindx ];

    threshold = .5; 
    rsp2 = pred_prob_pos>threshold;        
    accuracyi2(relation_to_learn) = sum(test_data_labels==rsp2)/length(test_data_labels); 
    corrPindxi2(relation_to_learn) = corr(pred_prob_pos,  humrating{relation_to_learn},'type','Pearson');
    corrSindxi2(relation_to_learn) = corr(pred_prob_pos,  humrating{relation_to_learn},'type','Spearman');
        
    fprintf('  BARTaccu = %f,  BARTcorrS = %f \n',(accuracyi2(relation_to_learn)), corrSindxi2(relation_to_learn)); 

    rst1(relation_to_learn,:) = [relation_to_learn, accuracyi2(relation_to_learn),...
        corrSindxi2(relation_to_learn) ];
        
end  %relation_to_learn ends

% display the result in the command window
save([outputrstpath 'rstsummary_converse.mat'], 'rst1');

toc;


