% use learned relation representations to compute relational similairity 
% between word pairs, and then solve for analogy problems in UCLA verbal
% analogy dataset.

function []=TestVebAnalogyfunc(learnrstpath,outputrstpath)
 
filerellab = {'1a','1b','1c','1d','1e','2a','2b','2c','2d','2e','2f','2g','2h','2i','2j',...
    '3a','3b','3c','3d','3e','3f','3g','3h','4a','4b','4c','4d','4e','4f','4g','4h',...
    '5a','5b','5c','5d','5e','5f','5g','5h','5i',...
    '6a','6b','6c','6d','6e','6f','6g','6h',...
    '7a','7b','7c','7d','7e','7f','7g','7h',...
    '8a','8b','8c','8d','8e','8f','8g','8h',...
    '9a','9b','9c','9d','9e','9f','9g','9h','9i',...
    '10a','10b','10c','10d','10e','10f'};
typelab = {1:5, 6:15,16:23,24:31,32:40,41:48,49:56,57:64,65:73,74:79};

% test data: UCLA verbal analogy dataset
filerel = {'category members','function','opposites','synonyms'};
datapath = '.\data\VerbAnalogy\';

num_sel_dims = 300; % feature dim of word vectors

% For variational method
max_iter = 500;                % maximum number of iterations
criterion = 0.00001;            % criterion for convergence


% read in learned relation representation
flipnum = 1; % no flip
for fi = 1:length(filerellab)
    load([learnrstpath sprintf('%s_mumat.mat', filerellab{fi})]);
    mumatsel{fi,1} = mumat(:,2); 
    load([learnrstpath sprintf('%s_sigma.mat', filerellab{fi})]); 
    sigma_postsel{fi,1} = sigma_post;  
    load([learnrstpath sprintf('%s_rsttemp.mat', filerellab{fi})]);%,'imptdimindx','selobjfdim'
    imptdimindxall{fi,1} = imptdimindx;
    selobjfdimall{fi,1} = selobjfdim; 
    dim_meansall{fi,1} = dim_means;
    pred_prob_posrel{fi,1} = pred_prob_pos;
end;


% read in all word pairs for each relation
for feature_to_learn = 1:length(filerel)

    %% load word vec
    datavalprot = [];
    load([datapath  filerel{feature_to_learn} '.txt.mat'],'allData'); 
    datavalprot=cell2mat(allData(:,2:301));
    Gpairnum(feature_to_learn) = size(datavalprot,1)/6;  %the first two AB pair, the second two CD pair, the last two CD' pair

    % generate AB/CD/CD'

       seq = 1:6; % 1:6
%      seq = [1 2 3 4 5 2]    
    for si = 1:Gpairnum(feature_to_learn)
        all_test_vecs{feature_to_learn}((si-1)*3+1,:) = [datavalprot((si-1)*6+seq(1),:) datavalprot((si-1)*6+seq(2),:)];
        all_test_names{(si-1)*3+1,feature_to_learn} = strcat(allData{(si-1)*6+seq(1),1}, ':', allData{(si-1)*6+seq(2),1});
        all_test_vecs{feature_to_learn}((si-1)*3+2,:) = [datavalprot((si-1)*6+seq(3),:) datavalprot((si-1)*6+seq(4),:)];
        all_test_names{(si-1)*3+2,feature_to_learn} = strcat(allData{(si-1)*6+seq(3),1}, ':', allData{(si-1)*6+seq(4),1});
        all_test_vecs{feature_to_learn}((si-1)*3+3,:) = [datavalprot((si-1)*6+seq(5),:) datavalprot((si-1)*6+seq(6),:)];
        all_test_names{(si-1)*3+3,feature_to_learn} = strcat(allData{(si-1)*6+seq(5),1}, ':', allData{(si-1)*6+seq(6),1});        
    end;

    test_data = all_test_vecs{feature_to_learn};
    % compute difference per feature
    t_data_diff = []; test_data_diff = []; t_scoreall=[];
    t_data_diff = ( test_data(:, 1:num_sel_dims)-test_data(:, num_sel_dims+1:end));  %abs
    
    % sort feature dim according difference
    for ti = 1:size(t_data_diff,1)
        temp1 = t_data_diff(ti, 1:num_sel_dims);
        [sorttemp1,sortindxt(ti,:)] = sort(temp1);
        test_data_diff(ti,:) = [temp1,sorttemp1];
    end;
                
    numfeats = num_sel_dims;
    for ti = 1:size(test_data,1)
         test_data2raw{feature_to_learn}(ti,:) = [test_data(ti,1:numfeats) test_data(ti,sortindxt(ti,:)) test_data(ti,numfeats+1:end) test_data(ti,numfeats+sortindxt(ti,:))];
    end;    
    
    totalobjfdim = size(test_data2raw,2); 
 
end;

% % % calculate post. prob.
for flipi = 1:flipnum
    for ti = 1:length(filerel)  % test dataset
        for fi = 1:length(filerellab)  % relation mu
            imptdimindx = [];
            imptdimindx = imptdimindxall{fi,flipi}; 

            disp([ti fi ]); 
            selobjfdim = [imptdimindx; numfeats*2+imptdimindx];
            test_data2 = [test_data2raw{ti}(:,selobjfdim) ];
            num_test_pairs = size(test_data2,1);

            % centering
            dim_means = dim_meansall{fi,flipi};%
            test_data2 = test_data2 - repmat(dim_means, size(test_data2,1), 1);

            mu_post = mumatsel{fi,flipi} ;
            if det(sigma_postsel{fi,flipi})==Inf
                fprintf('warning: %d: %s covmat det is Inf\n',flipi,filerellab{fi});
                %sigma_postsel{fi} = diag(diag(sigma_postsel{fi}));
            end;

            sigma_inv_post = pinv(sigma_postsel{fi,flipi});


            for test = 1 : num_test_pairs
                pair_vec_pos = test_data2(test, :);
                pred_prob_pos1{ti}(test,(flipi-1)*length(filerellab)+fi) = compute_score(pair_vec_pos', 1, mu_post, sigma_inv_post, max_iter, criterion);
            end; 
        end; % fi ends
    end; % ti ends
end;  % flipi ends



for ti = 1:length(typelab)
    all_rellabel(typelab{ti}) = ti;
end;

% concatenate
pred_prob_pos1temp = pred_prob_pos1;
% 
   
for ti = 1:length(filerel)  % test dataset
    for ci = 1:Gpairnum(ti) 
        dataci0 = pred_prob_pos1temp{ti}((ci-1)*3+1:ci*3,:);
        
        % normalized post prob
        for i = 1:3
            dataci(i,:) = dataci0(i,:)/sum(dataci0(i,:)); 
        end;
               
        distance_pair{ti}(ci,1) = pdist(dataci(1:2,:),'cosine'); 
        distance_pair{ti}(ci,2) = pdist(dataci([1 3],:),'cosine'); 
        
       
        if distance_pair{ti}(ci,1)<distance_pair{ti}(ci,2)
            decision(ci) = 1;
        else
            decision(ci) = 0;
        end;     
        
        fprintf('\n%d %d: %d %5.3f %5.3f ',ti, ci,decision(ci),distance_pair{ti}(ci,1),distance_pair{ti}(ci,2));
  
    end;
    
    accuracy(ti) = mean(decision);    
end; % ti ends

save([outputrstpath 'rst.UCLAAnalogy.mat'],'accuracy','distance_pair','pred_prob_pos1');
accuracy

